from collections.abc import Sequence
from enum import Enum
from typing import Literal
from typing_extensions import TypeAlias

import numpy as np
import numpy.typing as npt
import tensorflow as tf
from tensorflow._aliases import DTypeLike, ScalarTensorCompatible, ShapeLike
from tensorflow.python.trackable import autotrackable

class Algorithm(Enum):
    PHILOX = 1
    THREEFRY = 2
    AUTO_SELECT = 3

_Alg: TypeAlias = Literal[Algorithm.PHILOX, Algorithm.THREEFRY, Algorithm.AUTO_SELECT, "philox", "threefry", "auto_select"]

class Generator(autotrackable.AutoTrackable):
    @classmethod
    def from_state(cls, state: tf.Variable, alg: _Alg | None) -> Generator: ...
    @classmethod
    def from_seed(cls, seed: int, alg: _Alg | None = None) -> Generator: ...
    @classmethod
    def from_non_deterministic_state(cls, alg: _Alg | None = None) -> Generator: ...
    @classmethod
    def from_key_counter(
        cls, key: ScalarTensorCompatible, counter: Sequence[ScalarTensorCompatible], alg: _Alg | None
    ) -> Generator: ...
    def __init__(self, copy_from: Generator | None = None, state: tf.Variable | None = None, alg: _Alg | None = None) -> None: ...
    def reset(self, state: tf.Variable) -> None: ...
    def reset_from_seed(self, seed: int) -> None: ...
    def reset_from_key_counter(self, key: ScalarTensorCompatible, counter: tf.Variable) -> None: ...
    @property
    def state(self) -> tf.Variable: ...
    @property
    def algorithm(self) -> int: ...
    @property
    def key(self) -> ScalarTensorCompatible: ...
    def skip(self, delta: int) -> tf.Tensor: ...
    def normal(
        self,
        shape: tf.Tensor | Sequence[int],
        mean: ScalarTensorCompatible = 0.0,
        stddev: ScalarTensorCompatible = 1.0,
        dtype: DTypeLike = ...,
        name: str | None = None,
    ) -> tf.Tensor: ...
    def truncated_normal(
        self,
        shape: ShapeLike,
        mean: ScalarTensorCompatible = 0.0,
        stddev: ScalarTensorCompatible = 1.0,
        dtype: DTypeLike = ...,
        name: str | None = None,
    ) -> tf.Tensor: ...
    def uniform(
        self,
        shape: ShapeLike,
        minval: ScalarTensorCompatible = 0,
        maxval: ScalarTensorCompatible | None = None,
        dtype: DTypeLike = ...,
        name: str | None = None,
    ) -> tf.Tensor: ...
    def uniform_full_int(self, shape: ShapeLike, dtype: DTypeLike = ..., name: str | None = None) -> tf.Tensor: ...
    def binomial(
        self, shape: ShapeLike, counts: tf.Tensor, probs: tf.Tensor, dtype: DTypeLike = ..., name: str | None = None
    ) -> tf.Tensor: ...
    def make_seeds(self, count: int = 1) -> tf.Tensor: ...
    def split(self, count: int = 1) -> list[Generator]: ...

def all_candidate_sampler(
    true_classes: tf.Tensor, num_true: int, num_sampled: int, unique: bool, seed: int | None = None, name: str | None = None
) -> tuple[tf.Tensor, tf.Tensor, tf.Tensor]: ...
def categorical(
    logits: tf.Tensor,
    num_samples: int | tf.Tensor,
    dtype: DTypeLike | None = None,
    seed: int | None = None,
    name: str | None = None,
) -> tf.Tensor: ...
def create_rng_state(seed: int, alg: _Alg) -> npt.NDArray[np.int64]: ...
def fixed_unigram_candidate_sampler(
    true_classes: tf.Tensor,
    num_true: int,
    num_sampled: int,
    unique: bool,
    range_max: int,
    vocab_file: str = "",
    distortion: float = 1.0,
    num_reserved_ids: int = 0,
    num_shards: int = 1,
    shard: int = 0,
    unigrams: Sequence[float] = (),
    seed: int | None = None,
    name: str | None = None,
) -> tuple[tf.Tensor, tf.Tensor, tf.Tensor]: ...
def fold_in(seed: tf.Tensor | Sequence[int], data: int, alg: _Alg = "auto_select") -> int: ...
def gamma(
    shape: tf.Tensor | Sequence[int],
    alpha: tf.Tensor | float | Sequence[float],
    beta: tf.Tensor | float | Sequence[float] | None = None,
    dtype: DTypeLike = ...,
    seed: int | None = None,
    name: str | None = None,
) -> tf.Tensor: ...
def get_global_generator() -> Generator: ...
def learned_unigram_candidate_sampler(
    true_classes: tf.Tensor,
    num_true: int,
    num_sampled: int,
    unique: bool,
    range_max: int,
    seed: int | None = None,
    name: str | None = None,
) -> tuple[tf.Tensor, tf.Tensor, tf.Tensor]: ...
def log_uniform_candidate_sampler(
    true_classes: tf.Tensor,
    num_true: int,
    num_sampled: int,
    unique: bool,
    range_max: int,
    seed: int | None = None,
    name: str | None = None,
) -> tuple[tf.Tensor, tf.Tensor, tf.Tensor]: ...
def normal(
    shape: ShapeLike,
    mean: ScalarTensorCompatible = 0.0,
    stddev: ScalarTensorCompatible = 1.0,
    dtype: DTypeLike = ...,
    seed: int | None = None,
    name: str | None = None,
) -> tf.Tensor: ...
def poisson(
    shape: ShapeLike, lam: ScalarTensorCompatible, dtype: DTypeLike = ..., seed: int | None = None, name: str | None = None
) -> tf.Tensor: ...
def set_global_generator(generator: Generator) -> None: ...
def set_seed(seed: int) -> None: ...
def shuffle(value: tf.Tensor, seed: int | None = None, name: str | None = None) -> tf.Tensor: ...
def split(seed: tf.Tensor | Sequence[int], num: int = 2, alg: _Alg = "auto_select") -> tf.Tensor: ...
def stateless_binomial(
    shape: ShapeLike,
    seed: tuple[int, int] | tf.Tensor,
    counts: tf.Tensor,
    probs: tf.Tensor,
    output_dtype: DTypeLike = ...,
    name: str | None = None,
) -> tf.Tensor: ...
def stateless_categorical(
    logits: tf.Tensor,
    num_samples: int | tf.Tensor,
    seed: tuple[int, int] | tf.Tensor,
    dtype: DTypeLike = ...,
    name: str | None = None,
) -> tf.Tensor: ...
def stateless_gamma(
    shape: ShapeLike,
    seed: tuple[int, int] | tf.Tensor,
    alpha: tf.Tensor,
    beta: tf.Tensor | None = None,
    dtype: DTypeLike = ...,
    name: str | None = None,
) -> tf.Tensor: ...
def stateless_normal(
    shape: tf.Tensor | Sequence[int],
    seed: tuple[int, int] | tf.Tensor,
    mean: float | tf.Tensor = 0.0,
    stddev: float | tf.Tensor = 1.0,
    dtype: DTypeLike = ...,
    name: str | None = None,
    alg: _Alg = "auto_select",
) -> tf.Tensor: ...
def stateless_parameterized_truncated_normal(
    shape: tf.Tensor | Sequence[int],
    seed: tuple[int, int] | tf.Tensor,
    means: float | tf.Tensor = 0.0,
    stddevs: float | tf.Tensor = 1.0,
    minvals: tf.Tensor | float = -2.0,
    maxvals: tf.Tensor | float = 2.0,
    name: str | None = None,
) -> tf.Tensor: ...
def stateless_poisson(
    shape: tf.Tensor | Sequence[int],
    seed: tuple[int, int] | tf.Tensor,
    lam: tf.Tensor,
    dtype: DTypeLike = ...,
    name: str | None = None,
) -> tf.Tensor: ...
def stateless_truncated_normal(
    shape: tf.Tensor | Sequence[int],
    seed: tuple[int, int] | tf.Tensor,
    mean: float | tf.Tensor = 0.0,
    stddev: float | tf.Tensor = 1.0,
    dtype: DTypeLike = ...,
    name: str | None = None,
    alg: _Alg = "auto_select",
) -> tf.Tensor: ...
def stateless_uniform(
    shape: tf.Tensor | Sequence[int],
    seed: tuple[int, int] | tf.Tensor,
    minval: float | tf.Tensor = 0,
    maxval: float | tf.Tensor | None = None,
    dtype: DTypeLike = ...,
    name: str | None = None,
    alg: _Alg = "auto_select",
) -> tf.Tensor: ...
def truncated_normal(
    shape: tf.Tensor | Sequence[int],
    mean: float | tf.Tensor = 0.0,
    stddev: float | tf.Tensor = 1.0,
    dtype: DTypeLike = ...,
    seed: int | None = None,
    name: str | None = None,
) -> tf.Tensor: ...
def uniform(
    shape: tf.Tensor | Sequence[int],
    minval: float | tf.Tensor = 0,
    maxval: float | tf.Tensor | None = None,
    dtype: DTypeLike = ...,
    seed: int | None = None,
    name: str | None = None,
) -> tf.Tensor: ...
def uniform_candidate_sampler(
    true_classes: tf.Tensor,
    num_true: int,
    num_sampled: int,
    unique: bool,
    range_max: int,
    seed: int | None = None,
    name: str | None = None,
) -> tuple[tf.Tensor, tf.Tensor, tf.Tensor]: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import Widget
from reportlab.lib.attrmap import *
from reportlab.lib.validators import *

__version__: Final[str]

class _Symbol(Widget):
    x: int
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeWidth: float
    def __init__(self) -> None: ...
    def demo(self): ...

class ETriangle(_Symbol):
    def __init__(self) -> None: ...
    def draw(self): ...

class RTriangle(_Symbol):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class Octagon(_Symbol):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class Crossbox(_Symbol):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    crossColor: Incomplete
    strokeColor: Incomplete
    crosswidth: int
    def __init__(self) -> None: ...
    def draw(self): ...

class Tickbox(_Symbol):
    x: int
    y: int
    size: int
    tickColor: Incomplete
    strokeColor: Incomplete
    fillColor: Incomplete
    tickwidth: int
    def __init__(self) -> None: ...
    def draw(self): ...

class SmileyFace(_Symbol):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class StopSign(_Symbol):
    x: int
    y: int
    size: int
    strokeColor: Incomplete
    fillColor: Incomplete
    stopColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class NoEntry(_Symbol):
    x: int
    y: int
    size: int
    strokeColor: Incomplete
    fillColor: Incomplete
    innerBarColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class NotAllowed(_Symbol):
    x: int
    y: int
    size: int
    strokeColor: Incomplete
    fillColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class NoSmoking(NotAllowed):
    def __init__(self) -> None: ...
    def draw(self): ...

class DangerSign(_Symbol):
    x: int
    y: int
    size: int
    strokeColor: Incomplete
    fillColor: Incomplete
    strokeWidth: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class YesNo(_Symbol):
    x: int
    y: int
    size: int
    tickcolor: Incomplete
    crosscolor: Incomplete
    testValue: int
    def __init__(self) -> None: ...
    def draw(self): ...
    def demo(self): ...

class FloppyDisk(_Symbol):
    x: int
    y: int
    size: int
    diskColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class ArrowOne(_Symbol):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    strokeWidth: int
    strokeColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class ArrowTwo(ArrowOne):
    x: int
    y: int
    size: int
    fillColor: Incomplete
    strokeWidth: int
    strokeColor: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...

class CrossHair(_Symbol):
    x: int
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeWidth: float
    innerGap: str
    def __init__(self) -> None: ...
    def draw(self): ...

def test() -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Literal

from networkx.utils.backends import _dispatchable

__all__ = [
    "is_graphical",
    "is_multigraphical",
    "is_pseudographical",
    "is_digraphical",
    "is_valid_degree_sequence_erdos_gallai",
    "is_valid_degree_sequence_havel_hakimi",
]

@_dispatchable
def is_graphical(sequence: Iterable[Incomplete], method: Literal["eg", "hh"] = "eg") -> bool: ...
@_dispatchable
def is_valid_degree_sequence_havel_hakimi(deg_sequence: Iterable[Incomplete]) -> bool: ...
@_dispatchable
def is_valid_degree_sequence_erdos_gallai(deg_sequence: Iterable[Incomplete]) -> bool: ...
@_dispatchable
def is_multigraphical(sequence: Iterable[Incomplete]) -> bool: ...
@_dispatchable
def is_pseudographical(sequence: Iterable[Incomplete]) -> bool: ...
@_dispatchable
def is_digraphical(in_sequence: Iterable[Incomplete], out_sequence: Iterable[Incomplete]) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Final

import requests
from auth0.rest_async import RequestsResponse
from auth0.types import RequestData, TimeoutType

UNKNOWN_ERROR: Final[str]

class RestClientOptions:
    telemetry: bool
    timeout: TimeoutType
    retries: int
    def __init__(self, telemetry: bool = True, timeout: TimeoutType = 5.0, retries: int = 3) -> None: ...

class RestClient:
    options: RestClientOptions
    jwt: str | None
    base_headers: dict[str, str]
    telemetry: bool
    timeout: TimeoutType
    def __init__(
        self, jwt: str | None, telemetry: bool = True, timeout: TimeoutType = 5.0, options: RestClientOptions | None = None
    ) -> None: ...
    def MAX_REQUEST_RETRIES(self) -> int: ...
    def MAX_REQUEST_RETRY_JITTER(self) -> int: ...
    def MAX_REQUEST_RETRY_DELAY(self) -> int: ...
    def MIN_REQUEST_RETRY_DELAY(self) -> int: ...
    def get(self, url: str, params: dict[str, Incomplete] | None = None, headers: dict[str, str] | None = None): ...
    def post(self, url: str, data: RequestData | None = None, headers: dict[str, str] | None = None): ...
    def file_post(self, url: str, data: RequestData | None = None, files: dict[str, Incomplete] | None = None): ...
    def patch(self, url: str, data: RequestData | None = None): ...
    def put(self, url: str, data: RequestData | None = None): ...
    def delete(self, url: str, params: dict[str, Incomplete] | None = None, data: RequestData | None = None): ...

class Response:
    def __init__(self, status_code: int, content, headers: Mapping[str, str]) -> None: ...
    def content(self): ...

class JsonResponse(Response):
    def __init__(self, response: requests.Response | RequestsResponse) -> None: ...

class PlainResponse(Response):
    def __init__(self, response: requests.Response | RequestsResponse) -> None: ...

class EmptyResponse(Response):
    def __init__(self, status_code: int) -> None: ...



import re
from _typeshed import Incomplete
from typing import Final
from zoneinfo import ZoneInfo

from dateparser.date import DateData

PATTERN: Final[re.Pattern[str]]

class FreshnessDateDataParser:
    def get_local_tz(self) -> ZoneInfo: ...
    def parse(self, date_string: str, settings) -> tuple[Incomplete | None, str | None]: ...
    def get_kwargs(self, date_string: str) -> dict[str, float]: ...
    def get_date_data(self, date_string: str, settings=None) -> DateData: ...

freshness_date_parser: FreshnessDateDataParser



from _typeshed import Incomplete
from typing import Final

class ClientConfigurationEndpoint:
    ENDPOINT_NAME: Final = "client_configuration"
    server: Incomplete
    claims_classes: list[type[Incomplete]]
    def __init__(self, server=None, claims_classes: list[type[Incomplete]] | None = None) -> None: ...
    def __call__(self, request): ...
    def create_configuration_response(self, request): ...
    def create_endpoint_request(self, request): ...
    def create_read_client_response(self, client, request): ...
    def create_delete_client_response(self, client, request): ...
    def create_update_client_response(self, client, request): ...
    def extract_client_metadata(self, request): ...
    def introspect_client(self, client): ...
    def generate_client_registration_info(self, client, request) -> None: ...
    def authenticate_token(self, request) -> None: ...
    def authenticate_client(self, request) -> None: ...
    def revoke_access_token(self, token, request) -> None: ...
    def check_permission(self, client, request) -> None: ...
    def delete_client(self, client, request) -> None: ...
    def update_client(self, client, client_metadata, request) -> None: ...
    def get_server_metadata(self) -> None: ...



from _typeshed import Incomplete, OpenBinaryMode, ReadableBuffer
from asyncio import AbstractEventLoop
from collections.abc import Generator, Iterable
from concurrent.futures import Executor
from tempfile import TemporaryDirectory
from typing import TypeVar

from aiofiles.base import AsyncBase as AsyncBase
from aiofiles.threadpool.utils import (
    cond_delegate_to_executor as cond_delegate_to_executor,
    delegate_to_executor as delegate_to_executor,
    proxy_property_directly as proxy_property_directly,
)

_T = TypeVar("_T")

class AsyncSpooledTemporaryFile(AsyncBase[_T]):
    def fileno(self) -> Generator[Incomplete]: ...
    def rollover(self) -> Generator[Incomplete]: ...
    async def close(self) -> None: ...
    async def flush(self) -> None: ...
    async def isatty(self) -> bool: ...
    async def read(self, n: int = ..., /) -> str | bytes: ...
    async def readline(self, limit: int | None = ..., /) -> str | bytes: ...
    async def readlines(self, hint: int = ..., /) -> list[str | bytes]: ...
    async def seek(self, offset: int, whence: int = ...) -> int: ...
    async def tell(self) -> int: ...
    async def truncate(self, size: int | None = ...) -> None: ...
    @property
    def closed(self) -> bool: ...
    @property
    def encoding(self) -> str: ...
    @property
    def mode(self) -> OpenBinaryMode: ...
    @property
    def name(self) -> str | bytes: ...
    @property
    def newlines(self) -> str: ...
    async def write(self, s: str | bytes | ReadableBuffer) -> int: ...
    async def writelines(self, iterable: Iterable[str | bytes | ReadableBuffer]) -> None: ...

class AsyncTemporaryDirectory:
    async def cleanup(self) -> None: ...
    @property
    def name(self) -> str | bytes: ...
    def __init__(
        self, file: TemporaryDirectory[Incomplete], loop: AbstractEventLoop | None, executor: Executor | None
    ) -> None: ...
    async def close(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from re import Pattern

RESERVED_NS: Pattern[str]

def post_order_traverse(node) -> Generator[Incomplete, Incomplete]: ...
def reverse_post_order_traverse(node) -> Generator[Incomplete, Incomplete]: ...
def breadth_first_traverse(node) -> Generator[Incomplete]: ...
def longest_common_subsequence(left_sequence, right_sequence, eqfn=...): ...

WHITESPACE: Incomplete

def cleanup_whitespace(text): ...
def getpath(element, tree=None): ...
def make_ascii_tree(element): ...



import datetime
from typing import ClassVar

from ..core import ChineseNewYearCalendar, IslamicMixin

class Malaysia(IslamicMixin, ChineseNewYearCalendar):
    MSIA_DEEPAVALI: ClassVar[dict[int, datetime.date]]
    MSIA_THAIPUSAM: ClassVar[dict[int, datetime.date]]



from _typeshed import Incomplete

from reportlab.graphics.barcode.common import Barcode
from reportlab.graphics.widgetbase import Widget

class _DMTXCheck:
    @classmethod
    def pylibdmtx_check(cls) -> None: ...

class DataMatrix(Barcode, _DMTXCheck):
    color: Incomplete
    bgColor: Incomplete
    def __init__(self, value: str = "", **kwds) -> None: ...
    @property
    def value(self): ...
    @value.setter
    def value(self, v) -> None: ...
    @property
    def size(self): ...
    @size.setter
    def size(self, v) -> None: ...
    @property
    def border(self): ...
    @border.setter
    def border(self, v) -> None: ...
    @property
    def x(self): ...
    @x.setter
    def x(self, v) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, v) -> None: ...
    @property
    def cellSize(self): ...
    @cellSize.setter
    def cellSize(self, v) -> None: ...
    @property
    def encoding(self): ...
    @encoding.setter
    def encoding(self, v) -> None: ...
    @property
    def anchor(self): ...
    @anchor.setter
    def anchor(self, v) -> None: ...
    def recalc(self) -> None: ...
    @property
    def matrix(self): ...
    @property  # type: ignore[misc]  # TODO: for mypy < 1.16
    def width(self): ...  # type: ignore[override]
    @property  # type: ignore[misc]  # TODO: for mypy < 1.16
    def height(self): ...  # type: ignore[override]
    @property
    def cellWidth(self): ...
    @property
    def cellHeight(self): ...
    def draw(self) -> None: ...

class DataMatrixWidget(Widget, _DMTXCheck):
    codeName: str
    value: Incomplete
    def __init__(self, value: str = "Hello Cruel World!", **kwds) -> None: ...
    def rect(self, x, y, w, h, fill: int = 1, stroke: int = 0) -> None: ...
    def saveState(self, *args, **kwds) -> None: ...
    restoreState = saveState
    setStrokeColor = saveState
    def setFillColor(self, c) -> None: ...
    def draw(self): ...



from _typeshed import Unused

from pyasn1.codec.ber import decoder
from pyasn1.type import univ
from pyasn1.type.tag import TagSet

__all__ = ["decode", "StreamingDecoder"]

class BooleanPayloadDecoder(decoder.AbstractSimplePayloadDecoder):
    protoComponent: univ.Boolean
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Unused = None,
        **options,
    ): ...

BitStringPayloadDecoder = decoder.BitStringPayloadDecoder
OctetStringPayloadDecoder = decoder.OctetStringPayloadDecoder
RealPayloadDecoder = decoder.RealPayloadDecoder

TAG_MAP: dict[TagSet, decoder.AbstractPayloadDecoder]
TYPE_MAP: dict[int, decoder.AbstractPayloadDecoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemDecoder(decoder.SingleItemDecoder):
    TAG_MAP: dict[TagSet, decoder.AbstractPayloadDecoder]
    TYPE_MAP: dict[int, decoder.AbstractPayloadDecoder]

class StreamingDecoder(decoder.StreamingDecoder):
    SINGLE_ITEM_DECODER: type[SingleItemDecoder]

class Decoder(decoder.Decoder):
    STREAMING_DECODER: type[StreamingDecoder]

decode: Decoder



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.utils import CustomDrawChanger
from reportlab.graphics.shapes import Drawing, Group
from reportlab.graphics.widgetbase import PropHolder, Widget
from reportlab.lib.attrmap import *

__version__: Final[str]

class Label(Widget):
    # TODO: This has more attributes.
    x: Incomplete
    y: Incomplete
    def __init__(self, **kw) -> None: ...
    @property
    def padding(self): ...
    @padding.setter
    def padding(self, p) -> None: ...
    def setText(self, text) -> None: ...
    def setOrigin(self, x, y) -> None: ...
    def demo(self) -> Drawing: ...
    def computeSize(self) -> None: ...
    def draw(self) -> Group: ...

class LabelDecorator:
    textAnchor: str
    boxAnchor: str
    def __init__(self) -> None: ...
    def decorate(self, l, L) -> None: ...
    def __call__(self, l) -> None: ...

isOffsetMode: Incomplete

class LabelOffset(PropHolder):
    posMode: str
    pos: int
    def __init__(self) -> None: ...

NoneOrInstanceOfLabelOffset: Incomplete

class PMVLabel(Label):
    def __init__(self, **kwds) -> None: ...

class BarChartLabel(PMVLabel):
    lineStrokeWidth: int
    lineStrokeColor: Incomplete
    fixedStart: Incomplete
    nudge: int
    def __init__(self, **kwds) -> None: ...

class NA_Label(BarChartLabel):
    text: str
    def __init__(self) -> None: ...

NoneOrInstanceOfNA_Label: Incomplete

class RedNegativeChanger(CustomDrawChanger):
    fillColor: Incomplete
    def __init__(self, fillColor=...) -> None: ...

class XLabel(Label):
    ddfKlass: Incomplete
    ddf: Incomplete
    def __init__(self, *args, **kwds) -> None: ...
    def computeSize(self) -> None: ...



from _typeshed import Incomplete
from logging import Logger
from typing import Final, Literal
from typing_extensions import Self, TypeAlias

_SampledTrue: TypeAlias = Literal[True, "1", 1]
_SampledFalse: TypeAlias = Literal[False, "0", 0]
_SampledUnknown: TypeAlias = Literal["?"]
_Sampled: TypeAlias = _SampledTrue | _SampledFalse | _SampledUnknown

log: Logger
ROOT: Final = "Root"
PARENT: Final = "Parent"
SAMPLE: Final = "Sampled"
SELF: Final = "Self"
HEADER_DELIMITER: Final = ";"

class TraceHeader:
    def __init__(
        self,
        root: str | None = None,
        parent: str | None = None,
        sampled: _Sampled | None = None,
        data: dict[str, Incomplete] | None = None,
    ) -> None: ...
    @classmethod
    def from_header_str(cls, header: str | None) -> Self: ...
    def to_header_str(self) -> str: ...
    @property
    def root(self) -> str | None: ...
    @property
    def parent(self) -> str | None: ...
    @property
    def sampled(self) -> Literal[1, 0, "?"] | None: ...
    @property
    def data(self) -> dict[str, Incomplete]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message
import tensorflow.core.framework.allocation_description_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TensorDescription(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    ALLOCATION_DESCRIPTION_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    """Data type of tensor elements"""
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """Shape of the tensor."""

    @property
    def allocation_description(self) -> tensorflow.core.framework.allocation_description_pb2.AllocationDescription:
        """Information about the size and allocator used for the data"""

    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        allocation_description: tensorflow.core.framework.allocation_description_pb2.AllocationDescription | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["allocation_description", b"allocation_description", "shape", b"shape"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal["allocation_description", b"allocation_description", "dtype", b"dtype", "shape", b"shape"],
    ) -> None: ...

global___TensorDescription = TensorDescription



from ..console_menu import ExitItem as ExitItem, MenuItem as MenuItem
from .command_item import CommandItem as CommandItem
from .external_item import ExternalItem as ExternalItem
from .function_item import FunctionItem as FunctionItem
from .selection_item import SelectionItem as SelectionItem
from .submenu_item import SubmenuItem as SubmenuItem

__all__ = ["CommandItem", "ExitItem", "ExternalItem", "FunctionItem", "MenuItem", "SelectionItem", "SubmenuItem"]



from typing import Literal, SupportsIndex, final, overload
from typing_extensions import Never, Self, disjoint_base

import numpy as np
from numpy.typing import NDArray

area: np.ufunc
boundary: np.ufunc
bounds: np.ufunc
box: np.ufunc
buffer: np.ufunc
build_area: np.ufunc
centroid: np.ufunc
clip_by_rect: np.ufunc
concave_hull: np.ufunc
constrained_delaunay_triangles: np.ufunc
contains: np.ufunc
contains_properly: np.ufunc
contains_xy: np.ufunc
convex_hull: np.ufunc
coverage_invalid_edges: np.ufunc
coverage_is_valid: np.ufunc
coverage_simplify: np.ufunc
coverage_union: np.ufunc
covered_by: np.ufunc
covers: np.ufunc
create_collection: np.ufunc
crosses: np.ufunc
delaunay_triangles: np.ufunc
destroy_prepared: np.ufunc
difference: np.ufunc
difference_prec: np.ufunc
disjoint: np.ufunc
disjoint_subset_union: np.ufunc
distance: np.ufunc
dwithin: np.ufunc
envelope: np.ufunc
equals: np.ufunc
equals_exact: np.ufunc
equals_identical: np.ufunc
extract_unique_points: np.ufunc
force_2d: np.ufunc
force_3d: np.ufunc
frechet_distance: np.ufunc
frechet_distance_densify: np.ufunc
from_geojson: np.ufunc
from_wkb: np.ufunc
from_wkt: np.ufunc
get_coordinate_dimension: np.ufunc
get_dimensions: np.ufunc
get_exterior_ring: np.ufunc
get_geometry: np.ufunc
get_interior_ring: np.ufunc
get_m: np.ufunc
get_num_coordinates: np.ufunc
get_num_geometries: np.ufunc
get_num_interior_rings: np.ufunc
get_num_points: np.ufunc
get_point: np.ufunc
get_precision: np.ufunc
get_srid: np.ufunc
get_type_id: np.ufunc
has_m: np.ufunc
get_x: np.ufunc
get_y: np.ufunc
get_z: np.ufunc
has_z: np.ufunc
hausdorff_distance: np.ufunc
hausdorff_distance_densify: np.ufunc
intersection: np.ufunc
intersection_all: np.ufunc
intersection_prec: np.ufunc
intersects: np.ufunc
intersects_xy: np.ufunc
is_ccw: np.ufunc
is_closed: np.ufunc
is_empty: np.ufunc
is_geometry: np.ufunc
is_missing: np.ufunc
is_prepared: np.ufunc
is_ring: np.ufunc
is_simple: np.ufunc
is_valid: np.ufunc
is_valid_input: np.ufunc
is_valid_reason: np.ufunc
length: np.ufunc
line_interpolate_point: np.ufunc
line_interpolate_point_normalized: np.ufunc
line_locate_point: np.ufunc
line_locate_point_normalized: np.ufunc
line_merge: np.ufunc
line_merge_directed: np.ufunc
linearrings: np.ufunc
linestrings: np.ufunc
make_valid: np.ufunc
make_valid_with_params: np.ufunc
maximum_inscribed_circle: np.ufunc
minimum_bounding_circle: np.ufunc
minimum_bounding_radius: np.ufunc
minimum_clearance: np.ufunc
minimum_clearance_line: np.ufunc
node: np.ufunc
normalize: np.ufunc
offset_curve: np.ufunc
orient_polygons: np.ufunc
oriented_envelope: np.ufunc
overlaps: np.ufunc
point_on_surface: np.ufunc
points: np.ufunc
polygonize: np.ufunc
polygonize_full: np.ufunc
polygons: np.ufunc
prepare: np.ufunc
relate: np.ufunc
relate_pattern: np.ufunc
remove_repeated_points: np.ufunc
reverse: np.ufunc
segmentize: np.ufunc
set_precision: np.ufunc
set_srid: np.ufunc
shared_paths: np.ufunc
shortest_line: np.ufunc
simplify: np.ufunc
simplify_preserve_topology: np.ufunc
snap: np.ufunc
symmetric_difference: np.ufunc
symmetric_difference_all: np.ufunc
symmetric_difference_prec: np.ufunc
to_geojson: np.ufunc
to_wkb: np.ufunc
to_wkt: np.ufunc
touches: np.ufunc
unary_union: np.ufunc
unary_union_prec: np.ufunc
union: np.ufunc
union_prec: np.ufunc
voronoi_polygons: np.ufunc
within: np.ufunc
geos_capi_version: tuple[int, int, int]
geos_capi_version_string: str
geos_version: tuple[int, int, int]
geos_version_string: str
registry: list[type[Geometry]]

@disjoint_base
class Geometry:
    def __hash__(self) -> int: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __ge__(self, other: Never, /) -> bool: ...
    def __gt__(self, other: Never, /) -> bool: ...
    def __le__(self, other: Never, /) -> bool: ...
    def __lt__(self, other: Never, /) -> bool: ...

@final
class STRtree:
    count: int
    def __new__(cls, geoms: NDArray[np.object_], node_capacity: SupportsIndex, /, **kwargs: object) -> Self: ...
    def dwithin(self, geoms: NDArray[np.object_], distances: NDArray[np.float64], /) -> NDArray[np.int64]: ...
    def nearest(self, geoms: NDArray[np.object_], /) -> NDArray[np.int64]: ...
    def query(self, geoms: NDArray[np.object_], predicate: SupportsIndex, /) -> NDArray[np.int64]: ...
    def query_nearest(
        self, geoms: NDArray[np.object_], max_distance: float, exclusive: SupportsIndex, all_matches: SupportsIndex, /
    ) -> tuple[NDArray[np.int64], NDArray[np.float64]]: ...

class ShapelyError(Exception): ...
class GEOSException(ShapelyError): ...

def count_coordinates(geoms: NDArray[np.object_], /) -> int: ...
@overload
def get_coordinates(arr: NDArray[np.object_], include_z: bool, return_index: Literal[False], /) -> NDArray[np.float64]: ...
@overload
def get_coordinates(
    arr: NDArray[np.object_], include_z: bool, return_index: Literal[True], /
) -> tuple[NDArray[np.float64], NDArray[np.int64]]: ...
@overload
def get_coordinates(
    arr: NDArray[np.object_], include_z: bool, return_index: bool, /
) -> NDArray[np.float64] | tuple[NDArray[np.float64], NDArray[np.int64]]: ...
def set_coordinates(geoms: NDArray[np.object_], coords: NDArray[np.float64], /) -> NDArray[np.object_]: ...



from _typeshed import Incomplete
from datetime import datetime
from typing import TypedDict, type_check_only
from typing_extensions import Self

@type_check_only
class _ParamsDict(TypedDict, total=False):
    ach_mandate_text: str
    ach_mandate_accepted_at: str

class TransactionUsBankAccountRequest:
    parent: Incomplete
    def __init__(self, parent) -> None: ...
    def ach_mandate_text(self, ach_mandate_text: str) -> Self: ...
    def ach_mandate_accepted_at(self, ach_mandate_accepted_at: str | datetime) -> Self: ...
    def done(self): ...
    def to_param_dict(self) -> _ParamsDict: ...



from typing import Final

XK_Thai_kokai: Final = 0xDA1
XK_Thai_khokhai: Final = 0xDA2
XK_Thai_khokhuat: Final = 0xDA3
XK_Thai_khokhwai: Final = 0xDA4
XK_Thai_khokhon: Final = 0xDA5
XK_Thai_khorakhang: Final = 0xDA6
XK_Thai_ngongu: Final = 0xDA7
XK_Thai_chochan: Final = 0xDA8
XK_Thai_choching: Final = 0xDA9
XK_Thai_chochang: Final = 0xDAA
XK_Thai_soso: Final = 0xDAB
XK_Thai_chochoe: Final = 0xDAC
XK_Thai_yoying: Final = 0xDAD
XK_Thai_dochada: Final = 0xDAE
XK_Thai_topatak: Final = 0xDAF
XK_Thai_thothan: Final = 0xDB0
XK_Thai_thonangmontho: Final = 0xDB1
XK_Thai_thophuthao: Final = 0xDB2
XK_Thai_nonen: Final = 0xDB3
XK_Thai_dodek: Final = 0xDB4
XK_Thai_totao: Final = 0xDB5
XK_Thai_thothung: Final = 0xDB6
XK_Thai_thothahan: Final = 0xDB7
XK_Thai_thothong: Final = 0xDB8
XK_Thai_nonu: Final = 0xDB9
XK_Thai_bobaimai: Final = 0xDBA
XK_Thai_popla: Final = 0xDBB
XK_Thai_phophung: Final = 0xDBC
XK_Thai_fofa: Final = 0xDBD
XK_Thai_phophan: Final = 0xDBE
XK_Thai_fofan: Final = 0xDBF
XK_Thai_phosamphao: Final = 0xDC0
XK_Thai_moma: Final = 0xDC1
XK_Thai_yoyak: Final = 0xDC2
XK_Thai_rorua: Final = 0xDC3
XK_Thai_ru: Final = 0xDC4
XK_Thai_loling: Final = 0xDC5
XK_Thai_lu: Final = 0xDC6
XK_Thai_wowaen: Final = 0xDC7
XK_Thai_sosala: Final = 0xDC8
XK_Thai_sorusi: Final = 0xDC9
XK_Thai_sosua: Final = 0xDCA
XK_Thai_hohip: Final = 0xDCB
XK_Thai_lochula: Final = 0xDCC
XK_Thai_oang: Final = 0xDCD
XK_Thai_honokhuk: Final = 0xDCE
XK_Thai_paiyannoi: Final = 0xDCF
XK_Thai_saraa: Final = 0xDD0
XK_Thai_maihanakat: Final = 0xDD1
XK_Thai_saraaa: Final = 0xDD2
XK_Thai_saraam: Final = 0xDD3
XK_Thai_sarai: Final = 0xDD4
XK_Thai_saraii: Final = 0xDD5
XK_Thai_saraue: Final = 0xDD6
XK_Thai_sarauee: Final = 0xDD7
XK_Thai_sarau: Final = 0xDD8
XK_Thai_sarauu: Final = 0xDD9
XK_Thai_phinthu: Final = 0xDDA
XK_Thai_maihanakat_maitho: Final = 0xDDE
XK_Thai_baht: Final = 0xDDF
XK_Thai_sarae: Final = 0xDE0
XK_Thai_saraae: Final = 0xDE1
XK_Thai_sarao: Final = 0xDE2
XK_Thai_saraaimaimuan: Final = 0xDE3
XK_Thai_saraaimaimalai: Final = 0xDE4
XK_Thai_lakkhangyao: Final = 0xDE5
XK_Thai_maiyamok: Final = 0xDE6
XK_Thai_maitaikhu: Final = 0xDE7
XK_Thai_maiek: Final = 0xDE8
XK_Thai_maitho: Final = 0xDE9
XK_Thai_maitri: Final = 0xDEA
XK_Thai_maichattawa: Final = 0xDEB
XK_Thai_thanthakhat: Final = 0xDEC
XK_Thai_nikhahit: Final = 0xDED
XK_Thai_leksun: Final = 0xDF0
XK_Thai_leknung: Final = 0xDF1
XK_Thai_leksong: Final = 0xDF2
XK_Thai_leksam: Final = 0xDF3
XK_Thai_leksi: Final = 0xDF4
XK_Thai_lekha: Final = 0xDF5
XK_Thai_lekhok: Final = 0xDF6
XK_Thai_lekchet: Final = 0xDF7
XK_Thai_lekpaet: Final = 0xDF8
XK_Thai_lekkao: Final = 0xDF9



class ArcIV:
    def __init__(self, key) -> None: ...
    def reset(self) -> None: ...
    def encode(self, S): ...

def encode(text, key): ...
def decode(text, key): ...

__all__ = ["ArcIV", "encode", "decode"]



from _typeshed import Incomplete
from collections.abc import Iterator
from typing import ClassVar

from setuptools.dist import Distribution

from .._distutils.command import sdist as orig

def walk_revctrl(dirname: str = "") -> Iterator[Incomplete]: ...

class sdist(orig.sdist):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    negative_opt: ClassVar[dict[str, str]]
    README_EXTENSIONS: ClassVar[list[str]]
    READMES: ClassVar[tuple[str, ...]]
    filelist: Incomplete
    def run(self) -> None: ...
    def initialize_options(self) -> None: ...
    def make_distribution(self) -> None: ...
    def prune_file_list(self) -> None: ...
    def check_readme(self) -> None: ...
    def make_release_tree(self, base_dir, files) -> None: ...
    def read_manifest(self) -> None: ...



from typing import Final, Literal

TOL: Final = 1e-10

def machine_accuracy() -> tuple[float, int]: ...
def get_ordinal_suffix(ordinal: float) -> Literal["st", "nd", "rd", "th"]: ...
def iint(number: float) -> int: ...
def main() -> None: ...



from _typeshed import IdentityFunction
from collections.abc import Callable, Sequence
from logging import Logger
from typing import Any, TypeVar

_R = TypeVar("_R")

logging_logger: Logger

def retry_call(
    f: Callable[..., _R],
    fargs: Sequence[Any] | None = None,
    fkwargs: dict[str, Any] | None = None,
    exceptions: type[Exception] | tuple[type[Exception], ...] = ...,
    tries: int = -1,
    delay: float = 0,
    max_delay: float | None = None,
    backoff: float = 1,
    jitter: tuple[float, float] | float = 0,
    logger: Logger | None = ...,
) -> _R: ...
def retry(
    exceptions: type[Exception] | tuple[type[Exception], ...] = ...,
    tries: int = -1,
    delay: float = 0,
    max_delay: float | None = None,
    backoff: float = 1,
    jitter: tuple[float, float] | float = 0,
    logger: Logger | None = ...,
) -> IdentityFunction: ...



from typing import Any, NamedTuple

class Operation(NamedTuple):
    request: Any
    result: Any
    response: Any

class Cursor:
    connection: Any
    get_operational_attributes: Any
    definition: Any
    attributes: Any
    controls: Any
    execution_time: Any
    entries: Any
    schema: Any
    def __init__(
        self,
        connection,
        object_def,
        get_operational_attributes: bool = False,
        attributes=None,
        controls=None,
        auxiliary_class=None,
    ) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, item): ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def match_dn(self, dn): ...
    def match(self, attributes, value): ...
    def remove(self, entry) -> None: ...
    @property
    def operations(self): ...
    @property
    def errors(self): ...
    @property
    def failed(self): ...

class Reader(Cursor):
    entry_class: Any
    attribute_class: Any
    entry_initial_status: Any
    sub_tree: Any
    base: Any
    dereference_aliases: Any
    validated_query: Any
    query_filter: Any
    def __init__(
        self,
        connection,
        object_def,
        base,
        query: str = "",
        components_in_and: bool = True,
        sub_tree: bool = True,
        get_operational_attributes: bool = False,
        attributes=None,
        controls=None,
        auxiliary_class=None,
    ) -> None: ...
    @property
    def query(self): ...
    @query.setter
    def query(self, value) -> None: ...
    @property
    def components_in_and(self): ...
    @components_in_and.setter
    def components_in_and(self, value) -> None: ...
    def clear(self) -> None: ...
    execution_time: Any
    entries: Any
    def reset(self) -> None: ...
    def search(self, attributes=None): ...
    def search_object(self, entry_dn=None, attributes=None): ...
    def search_level(self, attributes=None): ...
    def search_subtree(self, attributes=None): ...
    def search_paged(self, paged_size, paged_criticality: bool = True, generator: bool = True, attributes=None): ...

class Writer(Cursor):
    entry_class: Any
    attribute_class: Any
    entry_initial_status: Any
    @staticmethod
    def from_cursor(cursor, connection=None, object_def=None, custom_validator=None): ...
    @staticmethod
    def from_response(connection, object_def, response=None): ...
    dereference_aliases: Any
    def __init__(
        self,
        connection,
        object_def,
        get_operational_attributes: bool = False,
        attributes=None,
        controls=None,
        auxiliary_class=None,
    ) -> None: ...
    execution_time: Any
    def commit(self, refresh: bool = True): ...
    def discard(self) -> None: ...
    def new(self, dn): ...
    def refresh_entry(self, entry, tries: int = 4, seconds: int = 2): ...



from _typeshed import Incomplete, SupportsItems
from collections.abc import Iterable
from dataclasses import dataclass
from io import BytesIO
from typing import Literal, overload

from PIL import Image

from .drawing import DeviceGray, DeviceRGB
from .enums import (
    Align,
    CellBordersLayout,
    TableBordersLayout,
    TableCellFillMode,
    TableHeadingsDisplay,
    TableSpan,
    VAlign,
    WrapMode,
)
from .fonts import FontFace
from .fpdf import FPDF
from .image_datastructures import _TextAlign
from .util import Padding

DEFAULT_HEADINGS_STYLE: FontFace

class Table:
    rows: list[Row]

    def __init__(
        self,
        fpdf: FPDF,
        rows: Iterable[str] = (),
        *,
        # Keep in sync with `fpdf.fpdf.FPDF.table`:
        align: str | _TextAlign = "CENTER",
        v_align: str | VAlign = "MIDDLE",
        borders_layout: str | TableBordersLayout = ...,
        cell_fill_color: int | tuple[Incomplete, ...] | DeviceGray | DeviceRGB | None = None,
        cell_fill_mode: str | TableCellFillMode = ...,
        col_widths: int | tuple[int, ...] | None = None,
        first_row_as_headings: bool = True,
        gutter_height: float = 0,
        gutter_width: float = 0,
        headings_style: FontFace = ...,
        line_height: int | None = None,
        markdown: bool = False,
        text_align: str | _TextAlign | tuple[str | _TextAlign, ...] = "JUSTIFY",
        width: int | None = None,
        wrapmode: WrapMode = ...,
        padding: float | Padding | None = None,
        outer_border_width: float | None = None,
        num_heading_rows: int = 1,
        repeat_headings: TableHeadingsDisplay | int = 1,
        min_row_height=None,
    ) -> None: ...
    def row(
        self, cells: Iterable[str] = (), style: FontFace | None = None, v_align: VAlign | str | None = None, min_height=None
    ) -> Row: ...
    def render(self) -> None: ...

class Row:
    cells: list[Cell]
    style: FontFace
    v_align: VAlign | None
    min_height: Incomplete | None
    def __init__(
        self, table: Table, style: FontFace | None = None, v_align: VAlign | str | None = None, min_height=None
    ) -> None: ...
    @property
    def cols_count(self) -> int: ...
    @property
    def max_rowspan(self) -> int: ...
    def convert_spans(self, active_rowspans: SupportsItems[int, int]) -> tuple[dict[int, int], list[int]]: ...
    @overload
    def cell(
        self,
        text: str = "",
        align: str | Align | None = None,
        v_align: str | VAlign | None = None,
        style: FontFace | None = None,
        img: str | Image.Image | BytesIO | None = None,
        img_fill_width: bool = False,
        colspan: int = 1,
        rowspan: int = 1,
        padding: tuple[float, ...] | None = None,
        link: str | int | None = None,
        border: CellBordersLayout | int = ...,
    ) -> str: ...
    @overload
    def cell(
        self,
        text: TableSpan,
        align: str | Align | None = None,
        v_align: str | VAlign | None = None,
        style: FontFace | None = None,
        img: str | Image.Image | BytesIO | None = None,
        img_fill_width: bool = False,
        colspan: int = 1,
        rowspan: int = 1,
        padding: tuple[float, ...] | None = None,
        link: str | int | None = None,
        border: CellBordersLayout | int = ...,
    ) -> TableSpan: ...

@dataclass
class Cell:
    __slots__ = ("text", "align", "v_align", "style", "img", "img_fill_width", "colspan", "rowspan", "padding", "link", "border")
    text: str
    align: str | Align | None
    v_align: str | VAlign | None
    style: FontFace | None
    img: str | None
    img_fill_width: bool
    colspan: int
    rowspan: int
    padding: int | tuple[float, ...] | None
    link: str | int | None
    border: CellBordersLayout | None

    def write(self, text, align=None): ...

@dataclass(frozen=True)
class RowLayoutInfo:
    height: int
    pagebreak_height: float
    rendered_heights: dict[Incomplete, Incomplete]
    merged_heights: list[Incomplete]

@dataclass(frozen=True)
class RowSpanLayoutInfo:
    column: int
    start: int
    length: int
    contents_height: float

    def row_range(self) -> range: ...

def draw_box_borders(pdf: FPDF, x1, y1, x2, y2, border: str | Literal[0, 1], fill_color=None) -> None: ...



from __future__ import annotations

from typing_extensions import assert_type

from simplejson import JSONEncoder, dumps


class CustomEncoder(JSONEncoder):  # eventhough it does not have `extra_kw` arg.
    ...


# We are only testing `dumps` here, because they are all the same:
dumps([], extra_kw=True)  # type: ignore

# Ok:
assert_type(dumps([], cls=CustomEncoder, extra_kw=True), str)



from collections.abc import Collection
from logging import Logger
from typing_extensions import TypeAlias

from authlib.oauth2 import OAuth2Request
from authlib.oauth2.rfc6749 import AuthorizationEndpointMixin, BaseGrant, ClientMixin, TokenEndpointMixin

_ServerResponse: TypeAlias = tuple[int, str, list[tuple[str, str]]]

log: Logger

class AuthorizationCodeGrant(BaseGrant, AuthorizationEndpointMixin, TokenEndpointMixin):
    TOKEN_ENDPOINT_AUTH_METHODS: Collection[str]
    AUTHORIZATION_CODE_LENGTH: int
    RESPONSE_TYPES: Collection[str]
    GRANT_TYPE: str
    def validate_authorization_request(self) -> str: ...
    def create_authorization_response(self, redirect_uri: str, grant_user) -> _ServerResponse: ...
    def validate_token_request(self) -> None: ...
    def create_token_response(self) -> _ServerResponse: ...
    def generate_authorization_code(self) -> str: ...
    def save_authorization_code(self, code: str, request: OAuth2Request) -> None: ...
    def query_authorization_code(self, code: str, client: ClientMixin): ...
    def delete_authorization_code(self, authorization_code) -> None: ...
    def authenticate_user(self, authorization_code): ...

def validate_code_authorization_request(grant: AuthorizationCodeGrant) -> str: ...



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Generator
from typing import NamedTuple

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["k_edge_augmentation", "is_k_edge_connected", "is_locally_k_edge_connected"]

@_dispatchable
def is_k_edge_connected(G: Graph[_Node], k: int) -> bool: ...
@_dispatchable
def is_locally_k_edge_connected(G: Graph[_Node], s: _Node, t: _Node, k: int) -> bool: ...
@_dispatchable
def k_edge_augmentation(
    G: Graph[_Node],
    k: int,
    avail: set[tuple[int, int]] | set[tuple[int, int, float]] | SupportsGetItem[tuple[int, int], float] | None = None,
    weight: str | None = None,
    partial: bool = False,
) -> Generator[tuple[_Node, _Node], None, None]: ...
@_dispatchable
def partial_k_edge_augmentation(G: Graph[_Node], k, avail, weight: str | None = None): ...
@_dispatchable
def one_edge_augmentation(G: Graph[_Node], avail=None, weight: str | None = None, partial: bool = False): ...
@_dispatchable
def bridge_augmentation(G: Graph[_Node], avail=None, weight: str | None = None): ...

class MetaEdge(NamedTuple):
    meta_uv: Incomplete
    uv: Incomplete
    w: Incomplete

@_dispatchable
def unconstrained_one_edge_augmentation(G: Graph[_Node]): ...
@_dispatchable
def weighted_one_edge_augmentation(G: Graph[_Node], avail, weight: str | None = None, partial: bool = False): ...
@_dispatchable
def unconstrained_bridge_augmentation(G: Graph[_Node]): ...
@_dispatchable
def weighted_bridge_augmentation(G: Graph[_Node], avail, weight: str | None = None): ...
@_dispatchable
def collapse(G: Graph[_Node], grouped_nodes): ...
@_dispatchable
def complement_edges(G: Graph[_Node]): ...
@_dispatchable
def greedy_k_edge_augmentation(G: Graph[_Node], k, avail=None, weight: str | None = None, seed=None): ...



import datetime
from _typeshed import ConvertibleToFloat, ConvertibleToInt, SupportsKeysAndGetItem, Unused
from collections.abc import Iterable, Iterator
from enum import Enum
from re import Pattern
from typing import Any, ClassVar, Final, Literal, Protocol, SupportsIndex, overload, type_check_only
from typing_extensions import Self, TypeAlias

from .caselessdict import CaselessDict
from .parser import Parameters
from .parser_tools import ICAL_TYPE
from .timezone import tzid_from_dt as tzid_from_dt, tzid_from_tzinfo as tzid_from_tzinfo

__all__ = [
    "DURATION_REGEX",
    "TimeBase",
    "TypesFactory",
    "WEEKDAY_RULE",
    "tzid_from_dt",
    "vBinary",
    "vBoolean",
    "vCalAddress",
    "vCategory",
    "vDDDLists",
    "vDDDTypes",
    "vDate",
    "vDatetime",
    "vDuration",
    "vFloat",
    "vFrequency",
    "vGeo",
    "vInline",
    "vInt",
    "vMonth",
    "vPeriod",
    "vRecur",
    "vText",
    "vTime",
    "vUTCOffset",
    "vUri",
    "vWeekday",
    "tzid_from_tzinfo",
    "vSkip",
]

_PropType: TypeAlias = type[Any]  # any of the v* classes in this file
_PeriodTuple: TypeAlias = tuple[datetime.datetime, datetime.datetime | datetime.timedelta]
_AnyTimeType: TypeAlias = datetime.datetime | datetime.date | datetime.timedelta | datetime.time | _PeriodTuple

@type_check_only
class _vType(Protocol):
    def to_ical(self) -> bytes | str: ...

DURATION_REGEX: Final[Pattern[str]]
WEEKDAY_RULE: Final[Pattern[str]]

class vBinary:
    obj: str
    params: Parameters
    def __init__(self, obj: str | bytes) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> bytes: ...
    def __eq__(self, other: object) -> bool: ...

class vBoolean(int):
    BOOL_MAP: Final[CaselessDict[bool]]
    params: Parameters
    def __new__(cls, x: ConvertibleToInt = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> Literal[b"TRUE", b"FALSE"]: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> bool: ...

class vText(str):
    encoding: str
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...
    ALTREP: property
    LANGUAGE: property
    RELTYPE: property

class vCalAddress(str):
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding="utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...
    @property
    def email(self) -> str: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str) -> None: ...
    @name.deleter
    def name(self) -> None: ...
    CN: property
    CUTYPE: property
    DELEGATED_FROM: property
    DELEGATED_TO: property
    DIR: property
    LANGUAGE: property
    PARTSTAT: property
    ROLE: property
    RSVP: property
    SENT_BY: property

class vFloat(float):
    params: Parameters
    def __new__(cls, x: ConvertibleToFloat = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vInt(int):
    params: Parameters
    def __new__(cls, x: ConvertibleToInt = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vDDDLists:
    params: Parameters
    dts: list[vDDDTypes]
    def __init__(self, dt_list: Iterable[_AnyTimeType] | _AnyTimeType) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: str, timezone: str | datetime.timezone | None = None): ...
    def __eq__(self, other: object) -> bool: ...

class vCategory:
    cats: list[vText]
    params: Parameters
    def __init__(self, c_list: Iterable[ICAL_TYPE] | ICAL_TYPE, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    RANGE: property
    RELATED: property
    TZID: property

class TimeBase:
    params: Parameters
    ignore_for_equality: set[str]
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    RANGE: property
    RELATED: property
    TZID: property

class vDDDTypes(TimeBase):
    params: Parameters
    dt: _AnyTimeType
    def __init__(self, dt: _AnyTimeType) -> None: ...
    def to_ical(self) -> bytes: ...
    @overload
    @classmethod
    def from_ical(cls, ical: Self, timezone: Unused | None = None) -> _AnyTimeType: ...
    # Return type is one of vDuration, vPeriod, vDatetime, vDate, or vTime,
    # depending on the ical string.
    @overload
    @classmethod
    def from_ical(cls, ical: str, timezone: datetime.timezone | str | None = None) -> Any: ...

class vDate(TimeBase):
    dt: datetime.date
    params: Parameters
    def __init__(self, dt: datetime.date) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> datetime.date: ...

class vDatetime(TimeBase):
    dt: datetime.datetime
    params: Parameters
    def __init__(self, dt: datetime.datetime, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE, timezone: datetime.timezone | str | None = None) -> datetime.datetime: ...

class vDuration(TimeBase):
    td: datetime.timedelta
    params: Parameters
    def __init__(self, td: datetime.timedelta, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: str) -> datetime.timedelta: ...
    @property
    def dt(self) -> datetime.timedelta: ...

class vPeriod(TimeBase):
    params: Parameters
    start: datetime.datetime
    end: datetime.datetime
    by_duration: bool
    duration: datetime.timedelta
    def __init__(self, per: _PeriodTuple) -> None: ...
    def overlaps(self, other: vPeriod) -> bool: ...
    def to_ical(self) -> bytes: ...
    # Return type is a tuple of vDuration, vPeriod, vDatetime, vDate, or vTime,
    # depending on the ical string. If the ical string is formed according to
    # the iCalendar specification, this should always return a
    # (datetime, datetime) or a (datetime, timedelta) tuple, but this is not
    # enforced.
    @staticmethod
    def from_ical(ical: str, timezone: datetime.timezone | str | None = None) -> tuple[Any, Any]: ...
    @property
    def dt(self) -> _PeriodTuple: ...
    FBTYPE: property

class vWeekday(str):
    week_days: Final[CaselessDict[int]]
    weekday: Literal["SU", "MO", "TU", "WE", "TH", "FR", "SA"] | None
    relative: int | None
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vFrequency(str):
    frequencies: Final[CaselessDict[str]]
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vMonth(int):
    params: Parameters
    def __new__(cls, month: vMonth | str | int, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: vMonth | str | int) -> Self: ...
    @property
    def leap(self) -> bool: ...
    @leap.setter
    def leap(self, value: bool) -> None: ...

class vSkip(vText, Enum):
    OMIT = "OMIT"
    FORWARD = "FORWARD"
    BACKWARD = "BACKWARD"

    def __reduce_ex__(self, _p: Unused) -> tuple[Self, tuple[str]]: ...

# The type of the values depend on the key. Each key maps to a v* class, and
# the allowed types are the types that the corresponding v* class can parse.
class vRecur(CaselessDict[Iterable[Any] | Any]):
    params: Parameters
    frequencies: Final[list[str]]
    canonical_order: ClassVar[tuple[str, ...]]
    types: Final[CaselessDict[_PropType]]
    def __init__(
        self, *args, params: SupportsKeysAndGetItem[str, str] = {}, **kwargs: list[Any] | tuple[Any, ...] | Any
    ) -> None: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def parse_type(cls, key: str, values: str) -> list[Any]: ...  # Returns a list of v* objects
    @classmethod
    def from_ical(cls, ical: vRecur | str) -> Self: ...

class vTime(TimeBase):
    dt: datetime.time | datetime.datetime
    params: Parameters
    @overload
    def __init__(self, dt: datetime.time | datetime.datetime, /) -> None: ...
    # args are passed to the datetime.time() constructor
    @overload
    def __init__(
        self,
        hour: SupportsIndex = ...,
        minute: SupportsIndex = ...,
        second: SupportsIndex = ...,
        microsecond: SupportsIndex = ...,
        tzinfo: datetime.tzinfo | None = ...,
        /,
    ) -> None: ...
    def to_ical(self) -> str: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> datetime.time: ...

class vUri(str):
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vGeo:
    latitude: float
    longitude: float
    params: Parameters
    def __init__(self, geo: tuple[float | str, float | str], params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> str: ...
    @staticmethod
    def from_ical(ical: str) -> tuple[float, float]: ...
    def __eq__(self, other: _vType) -> bool: ...  # type: ignore[override]

class vUTCOffset:
    ignore_exceptions: bool
    td: datetime.timedelta
    params: Parameters
    def __init__(self, td: datetime.timedelta, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> str: ...
    @classmethod
    def from_ical(cls, ical: Self | ICAL_TYPE) -> datetime.timedelta: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...

class vInline(str):
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class TypesFactory(CaselessDict[_PropType]):
    all_types: tuple[_PropType, ...]
    types_map: CaselessDict[str]
    def for_property(self, name: str) -> _PropType: ...
    # value is str | bytes, depending on what the v* class supports
    def to_ical(self, name: str, value: Any) -> bytes: ...
    # value and return type depend on what the v* class supports
    def from_ical(self, name: str, value: Any) -> Any: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class DocumentUploadGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult: ...



"""Annex module loading pass for the Jac compiler.

This pass handles the discovery, loading, and attachment of annex modules to their base modules.
Annex modules are specialized extension files (.impl.jac, .test.jac) that provide
implementations or tests for a base module.

Annex files are discovered from:
- Same directory: foo.impl.jac for foo.jac
- Module-specific folder: foo.impl/bar.impl.jac for foo.jac
- Shared folder: impl/foo.impl.jac, test/foo.test.jac

This enables the separation of interface and implementation, as well as test code organization.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

import jaclang.pycore.unitree as uni
from jaclang.pycore.bccache import discover_annex_files
from jaclang.pycore.passes.transform import Transform

if TYPE_CHECKING:
    from jaclang.pycore.program import JacProgram


class JacAnnexPass(Transform[uni.Module, uni.Module]):
    """Handles loading and attaching of annex files (.impl.jac, .test.jac)."""

    def transform(self, ir_in: uni.Module) -> uni.Module:
        """Load and attach annex modules to the given module."""
        mod_path = ir_in.loc.mod_path
        if ir_in.stub_only or not mod_path.endswith(".jac") or ir_in.annexable_by:
            return ir_in

        self._load_annexes(self.prog, ir_in, mod_path)
        return ir_in

    def _load_annexes(
        self, jac_program: JacProgram, node: uni.Module, mod_path: str
    ) -> None:
        """Parse and attach annex modules to the node."""
        for path in discover_annex_files(mod_path, ".impl.jac"):
            if mod := jac_program.compile(file_path=path, no_cgen=True, minimal=True):
                node.impl_mod.append(mod)

        for path in discover_annex_files(mod_path, ".test.jac"):
            if mod := jac_program.compile(file_path=path, no_cgen=True, minimal=True):
                node.test_mod.append(mod)



prTable: dict[int, str]

def GetPropTagName(pt): ...

mapiErrorTable: dict[int, str]

def GetScodeString(hr): ...

ptTable: dict[int, str]

def GetMapiTypeName(propType, rawType: bool = ...): ...
def GetProperties(obj, propList): ...
def GetAllProperties(obj, make_tag_names: bool = ...): ...
def SetPropertyValue(obj, prop, val) -> None: ...
def SetProperties(msg, propDict) -> None: ...



"""Declaration-Implementation Matching Pass for the Jac compiler.

This pass connects declarations (Decls) of Archetypes and Abilities with their separate
implementations (Defs) in the AST. It:

1. Establishes links between declarations in the main module and their implementations
   in separate .impl.jac files
2. Validates parameter matching between ability declarations and their implementations
3. Ensures proper inheritance of symbol tables between declarations and implementations

This pass is essential for Jac's separation of interface and implementation, allowing
developers to define archetype and ability interfaces in one file while implementing
their behavior in separate files.
"""

import jaclang.pycore.unitree as uni
from jaclang.pycore.passes.transform import Transform
from jaclang.pycore.unitree import Symbol, UniScopeNode


class DeclImplMatchPass(Transform[uni.Module, uni.Module]):
    """Decls and Def matching pass."""

    def transform(self, ir_in: uni.Module) -> uni.Module:
        """Connect Decls and Defs."""
        self.cur_node = ir_in

        self.connect_impls(ir_in.sym_tab, ir_in.sym_tab)
        for impl_module in ir_in.impl_mod:
            self.connect_impls(impl_module.sym_tab, ir_in.sym_tab)

        return ir_in

    def defn_lookup(self, lookup: Symbol) -> uni.NameAtom | None:
        """Lookup a definition in a symbol table."""
        for defn in range(len(lookup.defn)):
            candidate = lookup.defn[len(lookup.defn) - (defn + 1)]
            if (
                isinstance(candidate.name_of, uni.AstImplNeedingNode)
                and candidate.name_of.needs_impl
            ):
                return candidate
        return None

    def connect_impls(
        self, source_sym_tab: UniScopeNode, target_sym_tab: UniScopeNode
    ) -> None:
        """Connect implementations from source symbol table to declarations in target symbol table.

        When source_sym_tab and target_sym_tab are the same, this connects within the same module.
        When different, it connects implementations from impl_mod to declarations in the main module.
        """
        # Process all symbols in the source symbol table
        for sym in source_sym_tab.names_in_scope.values():
            if not isinstance(sym.decl.name_of, uni.ImplDef):
                continue

            # Extract archetype references
            arch_refs = sym.sym_name.split(".")[1:]  # Remove the impl. prefix
            name_of_links: list[uni.NameAtom] = []  # to link archref names to decls

            # Look up the archetype in the target symbol table
            lookup = target_sym_tab.lookup(arch_refs[0])

            # Skip over local import name collisions
            if lookup and not isinstance(lookup.decl.name_of, uni.AstImplNeedingNode):
                lookup = (
                    target_sym_tab.parent_scope.lookup(arch_refs[0])
                    if target_sym_tab.parent_scope
                    else target_sym_tab.lookup(arch_refs[0])
                )

            decl_node = (
                self.defn_lookup(lookup)
                if len(arch_refs) == 1 and lookup
                else lookup.defn[-1]
                if lookup
                else None
            )
            name_of_links.append(decl_node) if decl_node else None

            for name in arch_refs[1:]:
                if decl_node:
                    lookup = (
                        decl_node.name_of.sym_tab.lookup(name, deep=False)
                        if decl_node.name_of.sym_tab
                        else None
                    )
                    decl_node = (
                        self.defn_lookup(lookup)
                        if len(arch_refs) == 1 and lookup
                        else lookup.defn[-1]
                        if lookup
                        else None
                    )
                    name_of_links.append(decl_node) if decl_node else None
                else:
                    break

            if not decl_node:
                # self.log_error(
                #     f"Implementation for '{sym.sym_name}' cannot be matched to any declaration.",
                #     sym.decl.name_of,
                # )
                continue
            elif isinstance(decl_node, uni.Ability) and decl_node.is_abstract:
                self.log_warning(
                    f"Abstract ability {decl_node.py_resolve_name()} should not have a definition.",
                    decl_node,
                )
                continue

            if not isinstance(
                valid_decl := decl_node.name_of, uni.AstImplNeedingNode
            ) or not (valid_decl.sym_tab and sym.decl.name_of.sym_tab):
                raise self.ice(
                    f"Expected AstImplNeedingNode, got {valid_decl.__class__.__name__}. Not possible."
                )

            # Ensure if it's an ability def impl, the parameters are matched.
            self.validate_params_match(sym, decl_node.name_of)

            valid_decl.body = sym.decl.name_of
            sym.decl.name_of.decl_link = valid_decl
            for idx, a in enumerate(sym.decl.name_of.target):
                if idx < len(name_of_links) and (link := name_of_links[idx]):
                    a.name_spec.name_of = link.name_of
                    if (link_sym := link.sym) is not None:
                        a.name_spec.sym = link_sym
            sym.decl.name_of.sym_tab.names_in_scope.update(
                valid_decl.sym_tab.names_in_scope
            )
            valid_decl.sym_tab.names_in_scope = sym.decl.name_of.sym_tab.names_in_scope

            # Re-resolve symbols in the impl body now that symbol tables are synchronized.
            # This is necessary because impl files are compiled before being linked to their
            # base module, so symbol resolution during initial compilation fails for symbols
            # like 'self' and archetype members.
            self._resolve_impl_body_symbols(sym.decl.name_of)

        # Process kid scopes recursively
        for source_scope in source_sym_tab.kid_scope:
            # If source and target are the same, process within the same scope
            if source_sym_tab is target_sym_tab:
                self.connect_impls(source_scope, source_scope)
            else:
                # Otherwise, try to find corresponding scopes by name
                for target_scope in target_sym_tab.kid_scope:
                    if source_scope.scope_name == target_scope.scope_name:
                        self.connect_impls(source_scope, target_scope)
                        break

    def validate_params_match(self, sym: Symbol, valid_decl: uni.AstSymbolNode) -> None:
        """Validate if the parameters match."""
        if (
            isinstance(valid_decl, uni.Ability)
            and isinstance(sym.decl.name_of, uni.ImplDef)
            and isinstance(valid_decl.signature, uni.FuncSignature)
            and isinstance(sym.decl.name_of.spec, uni.FuncSignature)
        ):
            params_decl = valid_decl.signature.params
            params_defn = sym.decl.name_of.spec.params

            decl_count = len(params_decl) if params_decl else 0
            defn_count = len(params_defn) if params_defn else 0

            # Check if the parameter count is matched.
            if defn_count != decl_count:
                self.log_error(
                    f"Parameter count mismatch for ability {sym.sym_name}.",
                    sym.decl.name_of.name_spec,
                )
                self.log_error(
                    f"From the declaration of {valid_decl.name_spec.sym_name}.",
                    valid_decl.name_spec,
                )
            elif params_decl and params_defn:
                # Copy the parameter names from the declaration to the definition.
                for idx in range(len(params_defn)):
                    if (par := params_decl[idx].parent) is not None:
                        loc_in_kid = par.kid.index(params_decl[idx])
                        par.kid[loc_in_kid] = params_defn[idx]
                    params_decl[idx] = params_defn[idx]

    def _resolve_impl_body_symbols(self, impl_def: uni.ImplDef) -> None:
        """Re-resolve symbols in an impl body after symbol table synchronization.

        When impl files are compiled, they don't have access to the base module's
        symbols (like 'self' or archetype members). After DeclImplMatchPass links
        the impl to its declaration and synchronizes symbol tables, we need to
        re-traverse the impl body to resolve these symbols.
        """
        # Get all Name nodes that aren't part of an AtomTrailer (standalone names)
        for name_node in impl_def.get_all_sub_nodes(uni.Name):
            # Skip if already resolved or if part of AtomTrailer (handled separately)
            if name_node.sym is not None:
                continue
            if isinstance(name_node.parent, uni.AtomTrailer):
                continue
            # Try to resolve the symbol
            if name_node.sym_tab:
                name_node.sym_tab.use_lookup(name_node)

        # Get all AtomTrailer nodes (for chains like self.attr)
        for trailer_node in impl_def.get_all_sub_nodes(uni.AtomTrailer):
            chain = trailer_node.as_attr_list
            # Only re-resolve if the first element in the chain is unresolved
            if chain and chain[0].sym is None and trailer_node.sym_tab:
                trailer_node.sym_tab.chain_use_lookup(chain)



from collections import OrderedDict
from re import Pattern
from typing import ClassVar
from xml.etree.ElementTree import Element

from markdown.blockparser import BlockParser
from markdown.blockprocessors import BlockProcessor
from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor
from markdown.postprocessors import Postprocessor
from markdown.treeprocessors import Treeprocessor

FN_BACKLINK_TEXT: str
NBSP_PLACEHOLDER: str
RE_REF_ID: Pattern[str]
RE_REFERENCE: Pattern[str]

class FootnoteExtension(Extension):
    unique_prefix: int
    found_refs: dict[str, int]
    used_refs: set[str]
    def __init__(self, **kwargs) -> None: ...
    parser: BlockParser
    md: Markdown
    footnote_order: list[str]
    footnotes: OrderedDict[str, str]
    def reset(self) -> None: ...
    def unique_ref(self, reference: str, found: bool = False) -> str: ...
    def findFootnotesPlaceholder(self, root: Element) -> tuple[Element, Element, bool] | None: ...
    def setFootnote(self, id: str, text: str) -> None: ...
    def addFootnoteRef(self, id: str) -> None: ...
    def get_separator(self) -> str: ...
    def makeFootnoteId(self, id: str) -> str: ...
    def makeFootnoteRefId(self, id: str, found: bool = False) -> str: ...
    def makeFootnotesDiv(self, root: Element) -> Element | None: ...

class FootnoteBlockProcessor(BlockProcessor):
    RE: ClassVar[Pattern[str]]
    footnotes: FootnoteExtension
    def __init__(self, footnotes: FootnoteExtension) -> None: ...
    def detectTabbed(self, blocks: list[str]) -> list[str]: ...
    def detab(self, block: str) -> str: ...  # type: ignore[override]

class FootnoteInlineProcessor(InlineProcessor):
    footnotes: FootnoteExtension
    def __init__(self, pattern: str, footnotes: FootnoteExtension) -> None: ...

class FootnotePostTreeprocessor(Treeprocessor):
    footnotes: FootnoteExtension
    def __init__(self, footnotes: FootnoteExtension) -> None: ...
    def add_duplicates(self, li: Element, duplicates: int) -> None: ...
    def get_num_duplicates(self, li: Element) -> int: ...
    def handle_duplicates(self, parent: Element) -> None: ...
    def run(self, root: Element) -> None: ...
    offset: int

class FootnoteTreeprocessor(Treeprocessor):
    footnotes: FootnoteExtension
    def __init__(self, footnotes: FootnoteExtension) -> None: ...
    def run(self, root: Element) -> None: ...

class FootnoteReorderingProcessor(Treeprocessor):
    footnotes: FootnoteExtension
    def __init__(self, footnotes: FootnoteExtension) -> None: ...
    def run(self, root: Element) -> None: ...
    def reorder_footnotes(self, parent: Element) -> None: ...

class FootnotePostprocessor(Postprocessor):
    footnotes: FootnoteExtension
    def __init__(self, footnotes: FootnoteExtension) -> None: ...

def makeExtension(**kwargs) -> FootnoteExtension: ...



SPAN_KIND: str
SPAN_KIND_RPC_CLIENT: str
SPAN_KIND_RPC_SERVER: str
SPAN_KIND_CONSUMER: str
SPAN_KIND_PRODUCER: str
SERVICE: str
ERROR: str
COMPONENT: str
SAMPLING_PRIORITY: str
PEER_SERVICE: str
PEER_HOSTNAME: str
PEER_ADDRESS: str
PEER_HOST_IPV4: str
PEER_HOST_IPV6: str
PEER_PORT: str
HTTP_URL: str
HTTP_METHOD: str
HTTP_STATUS_CODE: str
DATABASE_INSTANCE: str
DATABASE_STATEMENT: str
DATABASE_TYPE: str
DATABASE_USER: str
MESSAGE_BUS_DESTINATION: str



from _io import BufferedReader
from gzip import GzipFile
from io import FileIO, RawIOBase, TextIOWrapper
from typing_extensions import assert_type

BufferedReader(RawIOBase())

assert_type(TextIOWrapper(FileIO("")).buffer, FileIO)
assert_type(TextIOWrapper(FileIO(13)).detach(), FileIO)
assert_type(TextIOWrapper(GzipFile("")).buffer, GzipFile)



from _typeshed import Incomplete
from collections.abc import Generator, Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["node_attribute_xy", "node_degree_xy"]

@_dispatchable
def node_attribute_xy(
    G: Graph[_Node], attribute, nodes: Iterable[Incomplete] | None = None
) -> Generator[Incomplete, None, None]: ...
@_dispatchable
def node_degree_xy(
    G: Graph[_Node], x: str = "out", y: str = "in", weight: str | None = None, nodes: Iterable[Incomplete] | None = None
) -> Generator[Incomplete, None, None]: ...



import tkinter
from collections.abc import Callable
from typing import Any, Literal

from ._widget import ThemedWidget

class ThemedTk(tkinter.Tk, ThemedWidget):
    def __init__(
        self,
        # non-keyword-only args copied from tkinter.Tk
        screenName: str | None = None,
        baseName: str | None = None,
        className: str = "Tk",
        useTk: bool = True,
        sync: bool = False,
        use: str | None = None,
        theme: str | None = None,
        # fonts argument does nothing
        toplevel: bool | None = None,
        themebg: bool | None = None,
        background: bool | None = None,  # old alias for themebg
        gif_override: bool = False,
    ) -> None: ...
    def set_theme(self, theme_name: str, toplevel: bool | None = None, themebg: bool | None = None) -> None: ...
    # Keep this in sync with tkinter.Tk
    def config(  # type: ignore[override]
        self,
        kw: dict[str, Any] | None = None,
        *,
        themebg: bool | None = ...,
        toplevel: bool | None = ...,
        theme: str | None = ...,
        background: str = ...,
        bd: float | str = ...,
        bg: str = ...,
        border: float | str = ...,
        borderwidth: float | str = ...,
        cursor: tkinter._Cursor = ...,
        height: float | str = ...,
        highlightbackground: str = ...,
        highlightcolor: str = ...,
        highlightthickness: float | str = ...,
        menu: tkinter.Menu = ...,
        padx: float | str = ...,
        pady: float | str = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: float | str = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None: ...
    def cget(self, k: str) -> Any: ...
    def configure(  # type: ignore[override]
        self,
        kw: dict[str, Any] | None = None,
        *,
        themebg: bool | None = ...,
        toplevel: bool | None = ...,
        theme: str | None = ...,
        background: str = ...,
        bd: float | str = ...,
        bg: str = ...,
        border: float | str = ...,
        borderwidth: float | str = ...,
        cursor: tkinter._Cursor = ...,
        height: float | str = ...,
        highlightbackground: str = ...,
        highlightcolor: str = ...,
        highlightthickness: float | str = ...,
        menu: tkinter.Menu = ...,
        padx: float | str = ...,
        pady: float | str = ...,
        relief: Literal["raised", "sunken", "flat", "ridge", "solid", "groove"] = ...,
        takefocus: bool | Literal[0, 1, ""] | Callable[[str], bool | None] = ...,
        width: float | str = ...,
    ) -> dict[str, tuple[str, str, str, Any, Any]] | None: ...
    def __getitem__(self, k: str) -> Any: ...
    def __setitem__(self, k: str, v: Any) -> None: ...



from _typeshed import Incomplete

from braintree.exchange_rate_quote_input import ExchangeRateQuoteInput

class ExchangeRateQuoteRequest:
    quotes: list[ExchangeRateQuoteInput]
    def __init__(self) -> None: ...
    def add_exchange_rate_quote_input(self, attributes) -> ExchangeRateQuoteInput: ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...



from collections.abc import Sequence
from typing import Literal, TypeVar, overload

from tensorflow import RaggedTensor, Tensor
from tensorflow._aliases import StringTensorCompatible, TensorCompatible, UIntTensorCompatible
from tensorflow.dtypes import DType

_TensorOrRaggedTensor = TypeVar("_TensorOrRaggedTensor", Tensor, RaggedTensor)

@overload
def as_string(
    input: TensorCompatible,
    precision: int = -1,
    scientific: bool = False,
    shortest: bool = False,
    width: int = -1,
    fill: str = "",
    name: str | None = None,
) -> Tensor: ...
@overload
def as_string(
    input: RaggedTensor,
    precision: int = -1,
    scientific: bool = False,
    shortest: bool = False,
    width: int = -1,
    fill: str = "",
    name: str | None = None,
) -> RaggedTensor: ...
def bytes_split(input: TensorCompatible | RaggedTensor, name: str | None = None) -> RaggedTensor: ...
def format(
    template: str, inputs: TensorCompatible, placeholder: str = "{}", summarize: int = 3, name: str | None = None
) -> Tensor: ...
def join(inputs: Sequence[TensorCompatible | RaggedTensor], separator: str = "", name: str | None = None) -> Tensor: ...
@overload
def length(input: TensorCompatible, unit: Literal["BYTE", "UTF8_CHAR"] = "BYTE", name: str | None = None) -> Tensor: ...
@overload
def length(input: RaggedTensor, unit: Literal["BYTE", "UTF8_CHAR"] = "BYTE", name: str | None = None) -> RaggedTensor: ...
@overload
def lower(input: TensorCompatible, encoding: Literal["utf-8", ""] = "", name: str | None = None) -> Tensor: ...
@overload
def lower(input: RaggedTensor, encoding: Literal["utf-8", ""] = "", name: str | None = None) -> RaggedTensor: ...
def ngrams(
    data: StringTensorCompatible | RaggedTensor,
    ngram_width: int | Sequence[int],
    separator: str = " ",
    pad_values: tuple[int, int] | str | None = None,
    padding_width: int | None = None,
    preserve_short_sequences: bool = False,
    name: str | None = None,
) -> RaggedTensor: ...
def reduce_join(
    inputs: StringTensorCompatible | RaggedTensor,
    axis: int | None = None,
    keepdims: bool = False,
    separator: str = "",
    name: str | None = None,
) -> Tensor: ...
@overload
def regex_full_match(input: StringTensorCompatible, pattern: StringTensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def regex_full_match(input: RaggedTensor, pattern: StringTensorCompatible, name: str | None = None) -> RaggedTensor: ...
@overload
def regex_replace(
    input: StringTensorCompatible,
    pattern: StringTensorCompatible,
    rewrite: StringTensorCompatible,
    replace_global: bool = True,
    name: str | None = None,
) -> Tensor: ...
@overload
def regex_replace(
    input: RaggedTensor,
    pattern: StringTensorCompatible,
    rewrite: StringTensorCompatible,
    replace_global: bool = True,
    name: str | None = None,
) -> RaggedTensor: ...
def split(
    input: StringTensorCompatible | RaggedTensor,
    sep: StringTensorCompatible | None = None,
    maxsplit: int = -1,
    name: str | None = None,
) -> RaggedTensor: ...
@overload
def strip(input: StringTensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def strip(input: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def substr(
    input: StringTensorCompatible,
    pos: TensorCompatible,
    len: TensorCompatible,
    unit: Literal["BYTE", "UTF8_CHAR"] = "BYTE",
    name: str | None = None,
) -> Tensor: ...
@overload
def substr(
    input: RaggedTensor,
    pos: TensorCompatible,
    len: TensorCompatible,
    unit: Literal["BYTE", "UTF8_CHAR"] = "BYTE",
    name: str | None = None,
) -> RaggedTensor: ...
@overload
def to_hash_bucket(input: StringTensorCompatible, num_buckets: int, name: str | None = None) -> Tensor: ...
@overload
def to_hash_bucket(input: RaggedTensor, num_buckets: int, name: str | None = None) -> RaggedTensor: ...
@overload
def to_hash_bucket_fast(input: StringTensorCompatible, num_buckets: int, name: str | None = None) -> Tensor: ...
@overload
def to_hash_bucket_fast(input: RaggedTensor, num_buckets: int, name: str | None = None) -> RaggedTensor: ...
@overload
def to_hash_bucket_strong(
    input: StringTensorCompatible, num_buckets: int, key: Sequence[int], name: str | None = None
) -> Tensor: ...
@overload
def to_hash_bucket_strong(input: RaggedTensor, num_buckets: int, key: Sequence[int], name: str | None = None) -> RaggedTensor: ...
@overload
def to_number(input: StringTensorCompatible, out_type: DType = ..., name: str | None = None) -> Tensor: ...
@overload
def to_number(input: RaggedTensor, out_type: DType = ..., name: str | None = None) -> RaggedTensor: ...
@overload
def unicode_decode(
    input: StringTensorCompatible,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> Tensor | RaggedTensor: ...
@overload
def unicode_decode(
    input: RaggedTensor,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> RaggedTensor: ...
@overload
def unicode_decode_with_offsets(
    input: StringTensorCompatible,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> tuple[_TensorOrRaggedTensor, _TensorOrRaggedTensor]: ...
@overload
def unicode_decode_with_offsets(
    input: RaggedTensor,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> tuple[RaggedTensor, RaggedTensor]: ...
@overload
def unicode_encode(
    input: TensorCompatible,
    output_encoding: Literal["UTF-8", "UTF-16-BE", "UTF-32-BE"],
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> Tensor: ...
@overload
def unicode_encode(
    input: RaggedTensor,
    output_encoding: Literal["UTF-8", "UTF-16-BE", "UTF-32-BE"],
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> RaggedTensor: ...
@overload
def unicode_script(input: TensorCompatible, name: str | None = None) -> Tensor: ...
@overload
def unicode_script(input: RaggedTensor, name: str | None = None) -> RaggedTensor: ...
@overload
def unicode_split(
    input: StringTensorCompatible,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> Tensor | RaggedTensor: ...
@overload
def unicode_split(
    input: RaggedTensor,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> RaggedTensor: ...
@overload
def unicode_split_with_offsets(
    input: StringTensorCompatible,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> tuple[_TensorOrRaggedTensor, _TensorOrRaggedTensor]: ...
@overload
def unicode_split_with_offsets(
    input: RaggedTensor,
    input_encoding: str,
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    name: str | None = None,
) -> tuple[RaggedTensor, RaggedTensor]: ...
@overload
def unicode_transcode(
    input: StringTensorCompatible,
    input_encoding: str,
    output_encoding: Literal["UTF-8", "UTF-16-BE", "UTF-32-BE"],
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> Tensor: ...
@overload
def unicode_transcode(
    input: RaggedTensor,
    input_encoding: str,
    output_encoding: Literal["UTF-8", "UTF-16-BE", "UTF-32-BE"],
    errors: Literal["replace", "strict", "ignore"] = "replace",
    replacement_char: int = 65533,
    replace_control_characters: bool = False,
    name: str | None = None,
) -> RaggedTensor: ...
def unsorted_segment_join(
    inputs: StringTensorCompatible,
    segment_ids: UIntTensorCompatible,
    num_segments: UIntTensorCompatible,
    separator: str = "",
    name: str | None = None,
) -> Tensor: ...
@overload
def upper(input: TensorCompatible, encoding: Literal["utf-8", ""] = "", name: str | None = None) -> Tensor: ...
@overload
def upper(input: RaggedTensor, encoding: Literal["utf-8", ""] = "", name: str | None = None) -> RaggedTensor: ...



from argparse import ArgumentParser, Namespace
from typing import Any

from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger

from .._types import _ASGIAppType, _WSGIAppType

class BaseApplication:
    usage: str | None
    cfg: Config
    callable: _WSGIAppType | _ASGIAppType | None
    prog: str | None
    logger: GLogger

    def __init__(self, usage: str | None = None, prog: str | None = None) -> None: ...
    def do_load_config(self) -> None: ...
    def load_default_config(self) -> None: ...
    def init(self, parser: ArgumentParser, opts: Namespace, args: list[str]) -> dict[str, Any] | None: ...
    def load(self) -> _WSGIAppType | _ASGIAppType: ...
    def load_config(self) -> None: ...
    def reload(self) -> None: ...
    def wsgi(self) -> _WSGIAppType | _ASGIAppType: ...
    def run(self) -> None: ...

class Application(BaseApplication):
    def chdir(self) -> None: ...
    def get_config_from_filename(self, filename: str) -> dict[str, Any]: ...
    def get_config_from_module_name(self, module_name: str) -> dict[str, Any]: ...
    def load_config_from_module_name_or_filename(self, location: str) -> dict[str, Any]: ...
    def load_config_from_file(self, filename: str) -> dict[str, Any]: ...
    def load_config(self) -> None: ...
    def run(self) -> None: ...



from _typeshed import Incomplete
from typing_extensions import Self

from braintree.graphql.inputs.phone_input import PhoneInput

class CustomerSessionInput:
    def __init__(
        self,
        email: str | None = None,
        hashed_email: str | None = None,
        phone: PhoneInput | None = None,
        hashed_phone_number: str | None = None,
        device_fingerprint_id: str | None = None,
        paypal_app_installed: bool | None = None,
        venmo_app_installed: bool | None = None,
        user_agent: str | None = None,
    ) -> None: ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...
    @staticmethod
    def builder() -> Builder: ...

    class Builder:
        def __init__(self) -> None: ...
        def email(self, email: str) -> Self: ...
        def hashed_email(self, hashed_email: str) -> Self: ...
        def phone(self, phone: PhoneInput) -> Self: ...
        def hashed_phone_number(self, hashed_phone_number: str) -> Self: ...
        def device_fingerprint_id(self, device_fingerprint_id: str) -> Self: ...
        def paypal_app_installed(self, paypal_app_installed: bool) -> Self: ...
        def venmo_app_installed(self, venmo_app_installed: bool) -> Self: ...
        def user_agent(self, user_agent: str) -> Self: ...
        def build(self) -> CustomerSessionInput: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.areas import PlotArea
from reportlab.graphics.shapes import Drawing
from reportlab.graphics.widgetbase import PropHolder

__version__: Final[str]

class BarChartProperties(PropHolder):
    strokeColor: Incomplete
    fillColor: Incomplete
    strokeWidth: float
    symbol: Incomplete
    strokeDashArray: Incomplete
    def __init__(self) -> None: ...

class BarChart(PlotArea):
    def makeSwatchSample(self, rowNo, x, y, width, height): ...
    def getSeriesName(self, i, default=None): ...
    categoryAxis: Incomplete
    valueAxis: Incomplete
    barSpacing: int
    reversePlotOrder: int
    data: Incomplete
    useAbsolute: int
    barWidth: int
    groupSpacing: int
    barLabels: Incomplete
    barLabelFormat: Incomplete
    barLabelArray: Incomplete
    bars: Incomplete
    naLabel: Incomplete
    zIndexOverrides: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    def getSeriesOrder(self) -> None: ...
    def calcBarPositions(self) -> None: ...
    def makeBars(self): ...
    def draw(self): ...

class VerticalBarChart(BarChart): ...
class HorizontalBarChart(BarChart): ...

class _FakeGroup:
    def __init__(self, cmp=None) -> None: ...
    def add(self, what) -> None: ...
    def value(self): ...
    def sort(self) -> None: ...

class BarChart3D(BarChart):
    theta_x: float
    theta_y: float
    zDepth: Incomplete
    zSpace: Incomplete
    def calcBarPositions(self) -> None: ...
    def makeBars(self): ...

class VerticalBarChart3D(BarChart3D, VerticalBarChart): ...
class HorizontalBarChart3D(BarChart3D, HorizontalBarChart): ...

def sampleV0a(): ...
def sampleV0b(): ...
def sampleV0c(): ...
def sampleV1(): ...
def sampleV2a(): ...
def sampleV2b(): ...
def sampleV2c(): ...
def sampleV3(): ...
def sampleV4a(): ...
def sampleV4b(): ...
def sampleV4c(): ...
def sampleV4d(): ...

dataSample5: Incomplete

def sampleV5a(): ...
def sampleV5b(): ...
def sampleV5c1(): ...
def sampleV5c2(): ...
def sampleV5c3(): ...
def sampleV5c4(): ...
def sampleH0a(): ...
def sampleH0b(): ...
def sampleH0c(): ...
def sampleH1(): ...
def sampleH2a(): ...
def sampleH2b(): ...
def sampleH2c(): ...
def sampleH3(): ...
def sampleH4a(): ...
def sampleH4b(): ...
def sampleH4c(): ...
def sampleH4d(): ...
def sampleH5a(): ...
def sampleH5b(): ...
def sampleH5c1(): ...
def sampleH5c2(): ...
def sampleH5c3(): ...
def sampleH5c4(): ...
def sampleSymbol1(): ...
def sampleStacked1(): ...

class SampleH5c4(Drawing):
    def __init__(self, width: int = 400, height: int = 200, *args, **kw) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Metadata(google.protobuf.message.Message):
    """next: 2"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.bytes
    def __init__(self, *, name: builtins.bytes | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___Metadata = Metadata



from passlib.crypto._blowfish.base import BlowfishEngine as _BlowfishEngine

class BlowfishEngine(_BlowfishEngine):
    def encipher(self, l, r): ...
    def expand(self, key_words) -> None: ...

__all__ = ["BlowfishEngine"]



from _typeshed import FileDescriptorOrPath
from collections.abc import Callable
from os import _Environ
from typing import AnyStr

from .spawnbase import SpawnBase, _Logfile

PY3: bool

class spawn(SpawnBase[AnyStr]):
    use_native_pty_fork: bool
    STDIN_FILENO: int
    STDOUT_FILENO: int
    STDERR_FILENO: int
    str_last_chars: int
    cwd: FileDescriptorOrPath | None
    env: _Environ[str]
    echo: bool
    ignore_sighup: bool
    command: str
    args: list[str]
    name: str
    use_poll: bool
    def __init__(
        self,
        command: str,
        args: list[str] = [],
        timeout: float | None = 30,
        maxread: int = 2000,
        searchwindowsize: int | None = None,
        logfile: _Logfile | None = None,
        cwd: FileDescriptorOrPath | None = None,
        env: _Environ[str] | None = None,
        ignore_sighup: bool = False,
        echo: bool = True,
        preexec_fn: Callable[[], None] | None = None,
        encoding: str | None = None,
        codec_errors: str = "strict",
        dimensions: tuple[int, int] | None = None,
        use_poll: bool = False,
    ) -> None: ...
    child_fd: int
    closed: bool
    def close(self, force: bool = True) -> None: ...
    def isatty(self) -> bool: ...
    def waitnoecho(self, timeout: float | None = -1) -> None: ...
    def getecho(self) -> bool: ...
    def setecho(self, state: bool) -> None: ...
    def read_nonblocking(self, size: int = 1, timeout: float | None = -1) -> AnyStr: ...
    def write(self, s: str | bytes) -> None: ...
    def writelines(self, sequence: list[str | bytes]) -> None: ...
    def send(self, s: str | bytes) -> int: ...
    def sendline(self, s: str | bytes = "") -> int: ...
    def sendcontrol(self, char: str) -> int: ...
    def sendeof(self) -> None: ...
    def sendintr(self) -> None: ...
    @property
    def flag_eof(self) -> bool: ...
    @flag_eof.setter
    def flag_eof(self, value: bool) -> None: ...
    def eof(self) -> bool: ...
    def terminate(self, force: bool = False) -> bool: ...
    status: int | None
    exitstatus: int | None
    signalstatus: int | None
    terminated: bool
    def wait(self) -> int: ...
    def isalive(self) -> bool: ...
    def kill(self, sig: int) -> None: ...
    def getwinsize(self) -> tuple[int, int]: ...
    def setwinsize(self, rows, cols) -> None: ...
    def interact(
        self,
        escape_character="\x1d",
        input_filter: Callable[[AnyStr], AnyStr] | None = None,
        output_filter: Callable[[AnyStr], AnyStr] | None = None,
    ) -> None: ...

def spawnu(
    command: str,
    args: list[str] = [],
    timeout: float | None = 30,
    maxread: int = 2000,
    searchwindowsize: int | None = None,
    logfile: _Logfile | None = None,
    cwd: FileDescriptorOrPath | None = None,
    env: _Environ[str] | None = None,
    ignore_sighup: bool = False,
    echo: bool = True,
    preexec_fn: Callable[[], None] | None = None,
    encoding: str | None = "utf-8",
    codec_errors: str = "strict",
    dimensions: tuple[int, int] | None = None,
    use_poll: bool = False,
) -> spawn[str]: ...



from collections.abc import Collection, Mapping
from typing import Any, Literal
from typing_extensions import TypeAlias

import numpy as np
from networkx._typing import Array1D, Array2D, ArrayLike1D, Seed
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.typing import NDArray

__all__ = [
    "bipartite_layout",
    "circular_layout",
    "forceatlas2_layout",
    "kamada_kawai_layout",
    "random_layout",
    "rescale_layout",
    "rescale_layout_dict",
    "shell_layout",
    "spring_layout",
    "spectral_layout",
    "planar_layout",
    "fruchterman_reingold_layout",
    "spiral_layout",
    "multipartite_layout",
    "bfs_layout",
    "arf_layout",
]

_FloatArrayLike1D: TypeAlias = ArrayLike1D[float, np.number[Any]]  # Any because we don't care about the bit base

def random_layout(
    G: Graph[_Node],
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    seed: Seed | None = None,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float32]]: ...
def circular_layout(
    G: Graph[_Node], scale: float = 1, center: _FloatArrayLike1D | None = None, dim: int = 2, store_pos_as: str | None = None
) -> dict[_Node, Array1D[np.float64]]: ...
def shell_layout(
    G: Graph[_Node],
    nlist: Collection[Collection[_Node]] | None = None,
    rotate: float | None = None,
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def bipartite_layout(
    G: Graph[_Node],
    nodes: Collection[_Node] | None = None,
    align: Literal["vertical", "horizontal"] = "vertical",
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    aspect_ratio: float = ...,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def spring_layout(
    G: Graph[_Node],
    k: float | None = None,
    pos: Mapping[_Node, Collection[float]] | None = None,
    fixed: Collection[_Node] | None = None,
    iterations: int = 50,
    threshold: float = 0.0001,
    weight: str | None = "weight",
    scale: float | None = 1,
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    seed: Seed | None = None,
    store_pos_as: str | None = None,
    *,
    method: Literal["auto", "force", "energy"] = "auto",
    gravity: float = 1.0,
) -> dict[_Node, Array1D[np.float64]]: ...

fruchterman_reingold_layout = spring_layout

def kamada_kawai_layout(
    G: Graph[_Node],
    dist: Mapping[_Node, Mapping[_Node, float]] | None = None,
    pos: Mapping[_Node, Collection[float]] | None = None,
    weight: str | None = "weight",
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def spectral_layout(
    G: Graph[_Node],
    weight: str | None = "weight",
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def planar_layout(
    G: Graph[_Node], scale: float = 1, center: _FloatArrayLike1D | None = None, dim: int = 2, store_pos_as: str | None = None
) -> dict[_Node, Array1D[np.float64]]: ...
def spiral_layout(
    G: Graph[_Node],
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    dim: int = 2,
    resolution: float = 0.35,
    equidistant: bool = False,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def multipartite_layout(
    G: Graph[_Node],
    subset_key: str | Mapping[Any, Collection[_Node]] = "subset",  # layers can be "any" hashable
    align: Literal["vertical", "horizontal"] = "vertical",
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
def arf_layout(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]] | None = None,
    scaling: float = 1,
    a: float = 1.1,
    etol: float = 1e-06,
    dt: float = 0.001,
    max_iter: int = 1000,
    *,
    seed: Seed | None = None,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float32]]: ...
@_dispatchable
def forceatlas2_layout(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]] | None = None,
    *,
    max_iter: int = 100,
    jitter_tolerance: float = 1.0,
    scaling_ratio: float = 2.0,
    gravity: float = 1.0,
    distributed_action: bool = False,
    strong_gravity: bool = False,
    node_mass: Mapping[_Node, float] | None = None,
    node_size: Mapping[_Node, float] | None = None,
    weight: str | None = None,
    dissuade_hubs: bool = False,
    linlog: bool = False,
    seed: Seed | None = None,
    dim: int = 2,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float32]]: ...
def rescale_layout(pos: NDArray[np.number[Any]], scale: float = 1) -> Array2D[np.float64]: ...  # ignore the bit base
def rescale_layout_dict(pos: Mapping[_Node, Collection[float]], scale: float = 1) -> dict[_Node, Array1D[np.float64]]: ...
def bfs_layout(
    G: Graph[_Node],
    start: _Node,
    *,
    align: Literal["vertical", "horizontal"] = "vertical",
    scale: float = 1,
    center: _FloatArrayLike1D | None = None,
    store_pos_as: str | None = None,
) -> dict[_Node, Array1D[np.float64]]: ...
